import socket
from flag import GOODBOY_FLAG
from peer import ClientPeer
from braid import hex2braid,str2hex
from threading import Thread

class ClientError(Exception):
    def __init__(self):
        Exception.__init__(self)
        
class ServerError(Exception):
    def __init__(self):
        Exception.__init__(self)

class ClientSocket:

    """
    Client socket class
    """    
    
    def __init__(self, peer, server='127.0.0.1', port=31337):
        self.peer = peer
        self.server = server
        self.port = port
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        
    def __send(self, data):

        """
        Message format: [BYTE: size][BYTE[]: message]
        """
        
        size = len(data)
        if size>256:
            raise ServerError()
        self.socket.send(chr(size))
        self.socket.send(data)    
    
    def __recv(self):

        """
        Message format: [BYTE: size][BYTE[]: message]
        """

        s = self.socket.recv(1)
        if len(s)==0:
            raise ServerError()
        return self.socket.recv(ord(s)) 

    def handshake(self):
        try:
            self.socket.connect((self.server, self.port))
            pubkey = self.peer.getPublicKey()
            # read motd
            motd = self.__recv()
            print motd            
            
            # read publickey from server
            rawpubkey = self.__recv()

            # send our publickey
            self.__send(pubkey)

            # compute shared secret
            self.peer.computeSharedSecret(rawpubkey)
            data = self.__recv()

            # decrypt data
            plain = self.peer.decryptData(data)

            # displays data
            print '[+] Your leaked flag: %s' % plain
        except:
            pass
            
class ServerWorker(Thread):

    """
    Server worker: handles client connection
    """
    
    def __init__(self, peer, socket,allowed_pubkeys=[]):
        Thread.__init__(self)
        self.peer = peer
        self.socket = socket
        self.allowed_pubkeys = allowed_pubkeys
        self.motd = '[i] Welcome on Goofyleaks. Can I haz ur public kay ?'
    
    def __send(self, data):
        size = len(data)
        if size>256:
            raise ServerError()
        self.socket.send(chr(size))
        self.socket.send(data)    
    
    def __recv(self):
        s = self.socket.recv(1)
        if len(s)==0:
            raise ServerError()
        return self.socket.recv(ord(s))        
    
    def run(self):
        try:
            # send motd
            self.__send(self.motd)
            # send peer pubkey
            rawpubkey = self.peer.getPublicKey()
            self.__send(rawpubkey)
            # wait for pubkey
            pubkey = self.__recv()
            self.peer.computeSharedSecret(pubkey)
            if str2hex(pubkey) in self.allowed_pubkeys:
                # authenticated ! (public key is allowed)
                enc_data = self.peer.encryptData(GOODBOY_FLAG)
                self.__send(enc_data)
            else:
                # well, public key is not allowed, send error
                enc_data = self.peer.encryptData('##NOT ALLOWED##')
            self.__send(enc_data)
            self.socket.close()
        except ServerError:
            self.socket.close()        

class ServerSocket:

    """
    Server socket class (handles multiple client connections)
    """

    def __init__(self, peer,allowed_pubkeys=[]):
        self.allowed_pubkeys = allowed_pubkeys
        self.peer = peer
    
    def serve(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket.bind(('',31337))
        self.socket.listen(5)
        while 1:
            client,addr = self.socket.accept()
            t = ServerWorker(self.peer, client, self.allowed_pubkeys)
            t.start()
    