import socket
from threading import Thread
from braid import ClientBraidKey, ServerBraidKey, SharedSecret, createBraidFromRawKey, braid2hex
from Crypto.Cipher import Blowfish

class Peer:
    
    """
    Basic peer class (handles key exchange, un/ciphering)
    """

    def __init__(self, K, client,custom_key=None):
        if custom_key is None:
            if client:
                self.__key = ClientBraidKey(K)
            else:
                self.__key = ServerBraidKey(K)
        else:
            self.__key = custom_key
        self.__ss = SharedSecret(K, self.__key)
        self.secret = ''
        self.cipher = None
        
    def getPublicKey(self):
        return self.__key.getPublicKey()
        
    def computeSharedSecret(self, rawpubkey):
        self.secret = self.__ss.makeKey(createBraidFromRawKey(rawpubkey))
        self.cipher = Blowfish.new(self.secret, Blowfish.MODE_ECB)        

    def __pad_data(self, data):
        overflow = 8 - len(data)%8
        data = data+''.join([chr(overflow) for o in range(overflow)])
        return data
        
    def __unpad_data(self, data):
        overflow = ord(data[-1])
        return data[:-overflow]        
    
    def encryptData(self, data):
        if self.cipher:
            return self.cipher.encrypt(self.__pad_data(data))
        else:
            return None
    
    def decryptData(self, data):
        if self.cipher:
            return self.__unpad_data(self.cipher.decrypt(data))
        else:
            return None

class ClientPeer(Peer):
    """
    Client peer class
    """
    def __init__(self, K):
        Peer.__init__(self, K, True)
        
class ServerPeer(Peer):
    """
    Server peer class
    """
    def __init__(self, K):
        Peer.__init__(self, K, False)
